"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const eventemitter3_1 = require("eventemitter3");
const debug_1 = __importDefault(require("debug"));
const logger = debug_1.default('fun-ddr:events');
const emitter = new eventemitter3_1.EventEmitter();
function handle(name, handler) {
    logger(`Adding handler for event ${name}: ${handler.name}`);
    const handlerLog = logger.extend(name);
    emitter.on(name, async (ev) => {
        handlerLog('Handling started');
        try {
            await handler(ev);
        }
        catch (e) {
            handlerLog.extend('error')('Error in handler %O', e);
            return;
        }
        handlerLog('Handling finished');
    });
    return (id, data) => handler({
        id,
        data,
        name,
    });
}
exports.handle = handle;
function emit(name, ev) {
    logger(`Emitting event ${ev.name} for resource ${ev.id}`);
    emitter.emit(name, ev);
}
exports.emit = emit;
function emitImmediate(id, name, data) {
    emit(name, {
        id,
        name,
        data,
    });
}
exports.emitImmediate = emitImmediate;
